<?php
/**
 * Barcode abstract
 * 
 * @author Floris Luiten <www.florisluiten.nl>
 * @package DigitBarCodes
 * @version $Id: digitbarcodes (abstract.php), v1.01 2011/02/12 $
 * @copyright Please contact author for copyright information, but
 *            in general these rules apply:
 *            
 *            - Want to use this code to learn how to write your own
 *              classes or teach others how to code? Go ahead! Please 
 *              give me credits in a proper way. Also, I'd love to hear
 *              your from you how you used this code. 
 *              
 *            - Want to use this code to generate a library for personal
 *              use? Please, go ahead! I'd love to hear from you how 
 *              this code helped you and where you have applied it.
 *              
 *            - Want to make money off this code? Please don't do that
 *              without consulting me first. If you really don't want
 *              to contact me for whatever reason, donate a FAIR amount
 *              to charity, in relation to how much money you make.
 */
/**
 * Abstract class for barcodes containing digits only
 *
 * @package DigitBarCodes
 * @author Floris Luiten <www.florisluiten.nl>
 * @abstract
 */
abstract class digitBarCodes {
	/**
	 * The bitArray for the left guard
	 * @access protected
	 * @var array
	 */
	protected $leftGuard = array();
	/**
	 * The bitArray for the right guard
	 * @access protected
	 * @var array
	 */
	protected $rightGuard = array();
	/**
	 * A multi-dimensional array containing the center guards.
	 *
	 * @example array(4 => (1, 0, 1)) Will insert the guard 101 at 
	 *          position 4
	 * @access protected
	 * @var array
	 */
	protected $centerGuards = array();
	/**
	 * Declares the height of a guard
	 * @access protected
	 * @var integer
	 */
	protected $barcodeGuardLength = 50;
	/**
	 * Declares the height of a regular bar
	 * @access protected
	 * @var integer
	 */
	protected $barcodeLength = 40;
	
	/**
	 * Dummy function for returning the checksum based on the
	 * code. Returns false if no checksum could be generated
	 *
	 * @param integer The code
	 * @return integer|False The checksum as digit or False
	 *                       when no checksum could be created
	 */
	abstract public function calculateChecksum($code);
	
	/**
	 * Dummy function to generate a bitArray from the code.
	 * This bitArray can be used to render the actual
	 * barcode as an image.
	 * Returns False when no bitArray could be generated, 
	 * for example when the checksum failed or non-valid
	 * characters were found.
	 *
	 * @param string The code with or without the checksum
	 * @param array|False A bitArray or False otherwise
	 */
	abstract public function getBitarray($code);
	
	/**
	 * Dummy function to return the code if it's valid,
	 * if no checksum is provided the return will include this
	 *
	 * @param string The code to check
	 * @param string|False The valid code or False otherwise
	 */
	abstract public function getValidatedCode($code);
	
	/**
	 * Bitwise NOT on a bitArray (an array containing 0 or 1's)
	 *
	 * @param array The bitArray
	 * @return array The NOT bitArray
	 */
	public function bitArrayNOT($bits) {
		foreach ($bits AS $index => $bit) {
			$bits[$index] = 1 ^ $bit;
		}
		return $bits;
	}
	
	/**
	 * Convert a bitArray to an integer
	 *
	 * @param array The bitarray
	 * @return integer The integer
	 */
	public function bitArrayToInteger($bits) {
		$bitValue = 1;
		$sum = 0;
		
		foreach (array_reverse($bits) AS $index => $bit) {
			$sum += $bit * $bitValue;
			$bitValue *= 2;
		}
		return $sum;
	}
	
	/**
	 * Function generate the barcode based on the code. It first checks
	 * the checksum or creates it if it's missing.
	 *
	 * @param string The code (with or without the checksum)
	 * @param integer optional An optional size
	 */
	public function getBarcodeImage($code, $size=1) {
		$code = $this->getValidatedCode($code);
		if ($code == false) {
			return false;
		}
		return $this->generateBarcodeImage($this->getBitarray($code, false), $size);
	}
	
	/**
	 * Generates an image containing the barcode to use
	 * in printing or on screen. 
	 *
	 * @param array The bitarray containing the code
	 * @param integer optional An optional size (default: @default)
	 */
	public function generateBarcodeImage($bitArray, $size=1) {
		$imageWidth = count($bitArray, COUNT_RECURSIVE) - count($bitArray);
		
		$imageWidth += count($this->leftGuard);
		$imageWidth += count($this->centerGuards, COUNT_RECURSIVE) - count($this->centerGuards);
		$imageWidth += count($this->rightGuard);
		$imageWidth += $size;
		
		$image = imagecreate(($imageWidth * $size), ($this->barcodeGuardLength * $size));
		imagecolorallocate($image, 255, 255, 255); // White background
		$imColorBarcode = imagecolorallocate($image, 0, 0, 0); // Black color
		$xPos = 0;
		
		foreach ($this->leftGuard AS $bit) {
			if ($bit == 1) {
				$this->imagelinethick($image, $xPos+$size, 0, $xPos+$size, ($this->barcodeGuardLength * $size), $imColorBarcode, $size);
			}
			$xPos += $size;
		}
			
		foreach ($bitArray AS $index => $bitArray) {
			foreach ($bitArray AS $bit) {
				if ($bit == 1) {
					$this->imagelinethick($image, $xPos+$size, 0, $xPos+$size, ($this->barcodeLength * $size), $imColorBarcode, $size);
				}
				$xPos += $size;
			}
			
			if (isset($this->centerGuards[$index])) {
				foreach ($this->centerGuards[$index] AS $bit) {
					//echo 'Center guard bit ' . $bit . "<br>\n";
					if ($bit == 1) {
						$this->imagelinethick($image, $xPos+$size, 0, $xPos+$size, ($this->barcodeGuardLength * $size), $imColorBarcode, $size);
					}
					$xPos += $size;
				}
			}
		}
		
		foreach ($this->rightGuard AS $bit) {
			if ($bit == 1) {
				$this->imagelinethick($image, $xPos+$size, 0, $xPos+$size, ($this->barcodeGuardLength * $size), $imColorBarcode, $size);
			}
			$xPos += $size;
		}
		
		return $image;
	}
	
	/**
	 * Draw a line on the image of a specific width.
	 * 
	 * @link http://nl.php.net/manual/en/function.imageline.php
	 * 
	 * @param resource The image resource
	 * @param integer The x1
	 * @param integer The y1
	 * @param integer The x2
	 * @param integer The y2
	 * @param resource The color
	 * @param integer The thickness of the line
	 * @return boolean True on success, FALSE otherwise
	 */
	private function imagelinethick($image, $x1, $y1, $x2, $y2, $color, $thick = 1) {
		if ($thick == 1) {
			return imageline($image, $x1, $y1, $x2, $y2, $color);
		}
		$t = $thick / 2 - 0.5;
		if ($x1 == $x2 || $y1 == $y2) {
			return imagefilledrectangle($image, round(min($x1, $x2) - $t), round(min($y1, $y2) - $t), round(max($x1, $x2) + $t), round(max($y1, $y2) + $t), $color);
		}
		$k = ($y2 - $y1) / ($x2 - $x1);
		$a = $t / sqrt(1 + pow($k, 2));
		$points = array(
			round($x1 - (1+$k)*$a), round($y1 + (1-$k)*$a),
			round($x1 - (1-$k)*$a), round($y1 - (1+$k)*$a),
			round($x2 + (1+$k)*$a), round($y2 - (1-$k)*$a),
			round($x2 + (1-$k)*$a), round($y2 + (1+$k)*$a),
		);
		imagefilledpolygon($image, $points, 4, $color);
		return imagepolygon($image, $points, 4, $color);
	}
}
?>
