<?php
/**
 * Barcode EAN-13
 * 
 * @author Floris Luiten <www.florisluiten.nl>
 * @package DigitBarCodes
 * @version $Id: digitbarcodes (ean13.class.php), v1.01 2011/02/12 $
 * @copyright Please contact author for copyright information, but
 *            in general these rules apply:
 *            
 *            - Want to use this code to learn how to write your own
 *              classes or teach others how to code? Go ahead! Please 
 *              give me credits in a proper way. Also, I'd love to hear
 *              your from you how you used this code. 
 *              
 *            - Want to use this code to generate a library for personal
 *              use? Please, go ahead! I'd love to hear from you how 
 *              this code helped you and where you have applied it.
 *              
 *            - Want to make money off this code? Please don't do that
 *              without consulting me first. If you really don't want
 *              to contact me for whatever reason, donate a FAIR amount
 *              to charity, in relation to how much money you make.
 */

/**
 * Requires the abstract class
 */
require_once(dirname(__FILE__) . '/abstract.php');

/**
 * Class for EAN-13 codes
 * 
 * @package DigitBarCodes 
 * @abstract
 */
class EAN13Barcode extends digitBarCodes {
	protected $codeLengthNoCheckum = 12;
	protected $codeLength = 1;
	
	protected $leftGuard = array(1, 0, 1);
	protected $rightGuard = array(1, 0, 1);
	protected $centerGuards = array(5 => array(0, 1, 0, 1, 0));
	
	protected $encodeStartDigit = 1;
	
	protected $codeLookupTable = array(
		'L' => array(
			'0' => 
				array(0, 0, 0, 1, 1, 0, 1),
			'1' => 
				array(0, 0, 1, 1, 0, 0, 1),
			'2' => 
				array(0, 0, 1, 0, 0, 1, 1),
			'3' => 
				array(0, 1, 1, 1, 1, 0, 1),
			'4' => 
				array(0, 1, 0, 0, 0, 1, 1),
			'5' => 
				array(0, 1, 1, 0, 0, 0, 1),
			'6' => 
				array(0, 1, 0, 1, 1, 1, 1),
			'7' => 
				array(0, 1, 1, 1, 0, 1, 1),
			'8' => 
				array(0, 1, 1, 0, 1, 1, 1),
			'9' => 
				array(0, 0, 0, 1, 0, 1, 1)
			));
	
	/**
	 * Function for returning the checksum based on the input.
	 * Returns false if no checksum could be generated
	 *
	 * @param string The barcode
	 * @return integer The checksum as digit
	 */
	public function calculateChecksum($code) {
		if (!ctype_digit($code) OR !ctype_digit($code)) {
			return false;
		}
		$sum = 0;
		foreach (str_split(strrev($code)) AS $index => $digit) {
			//Thank god PHP thinks 1 & 0 == 0
			$sum += (1 & $index) ? $digit : $digit * 3;
		}
		return ($sum % 10 == 0) ? 0 : 10 - ($sum % 10);
	}
	
	/**
	 * Function to generate a bitArray from the code.
	 * This bitArray can be used to render the actual
	 * barcode as an image.
	 *
	 * @param string The code with or without the checksum
	 * @param boolean optional Include guards? (default: true)
	 * @return array|False A bitArray or False otherwise
	 */
	public function getBitarray($code, $includeGuards=true) {
		$code = $this->getValidatedCode($code);
		if ($code == false) {
			return false;
		}
				
		$return = array();
		if ($includeGuards) {
			$return[] = $this->leftGuard;
		}
		$structureMask = str_split($this->getStructureMask($code));
		
		foreach (str_split(substr($code, $this->encodeStartDigit)) AS $index => $digit) {
			$return[] = $this->encode($digit, $structureMask[$index]);
			if (isset($this->centerGuards[$index]) AND $includeGuards) {
				$return[] = $this->centerGuards[$index];
			}
		}
		if ($includeGuards) {
			$return[] = $this->rightGuard;
		}
		return $return;
	}
	
	/**
	 * Function to return the code if it's valid, if no if no checksum is
	 * provided the return will include this
	 *
	 * @param string The code to check
	 * @param string|False The valid code or False otherwise
	 */
	public function getValidatedCode($code) {
		if (!ctype_digit($code) OR !ctype_digit($code)) {
			return false;
		}
		if (strlen($code) == $this->codeLengthNoCheckum) {
			return $code . $this->calculateChecksum($code);
		} elseif (strlen($code) == ($this->codeLengthNoCheckum + $this->codeLength)) {
			if ($this->calculateChecksum(substr($code, 0, (0 -  $this->codeLength))) != substr($code, (0 -  $this->codeLength))) {
				return false;
			}
			return $code;
		}
		return false;
	}
	
	/**
	 * Returns the mask to use for the EAN-13 code
	 *
	 * @param string The code (with or without the checksum)
	 * @return string The mask to use
	 */
	protected function getStructureMask($code) {
		if (!ctype_digit($code)) {
			return false;
		}
		switch (substr($code, 0, 1)) {
			case '0':
				return 'LLLLLLRRRRRR';
			case '1':
				return 'LLGLGGRRRRRR';
			case '2':
				return 'LLGGLGRRRRRR';
			case '3':
				return 'LLGGGLRRRRRR';
			case '4':
				return 'LGLLGGRRRRRR';
			case '5':
				return 'LGGLLGRRRRRR';
			case '6':
				return 'LGGGLLRRRRRR';
			case '7':
				return 'LGLGLGRRRRRR';
			case '8':
				return 'LGLGGLRRRRRR';
			case '9':
				return 'LGGLGLRRRRRR';
		}
		return false; //Even though we can never actually reach this
	}
	
	/**
	 * Returns the L/G/R-code for the integer for usage with EAN-13
	 *
	 * @param integer The integer to encode
	 * @param string The codebase (R/G/L)
	 * @return integer The encoded integer
	 */
	protected function encode($integer, $codeBase) {
		if ($codeBase == 'R') {
			return $this->bitArrayNOT($this->codeLookupTable['L'][$integer]);
		} elseif ($codeBase == 'G') {
			return array_reverse($this->bitArrayNOT($this->codeLookupTable['L'][$integer]));
		} else {
			return $this->codeLookupTable['L'][$integer];
		}
	}
}
?>
